#!/usr/bin/python

##############################################################################
# Emilio Schapira
# Copyright (C) 2003 Advanced Interface Technologies, Inc.
# http://www.advancedinterfaces.com
# http://sourceforge.net/projects/pycron/
# 
# This program is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 2 of the License, or (at your
# option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
# for more details.
# 
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##############################################################################

"""
usage: cron [crontab_file_name [log_file_name [pid_file_name]]]
       This is a minimal cron clone that can be used in windows.
       It writes a log in the current directory, and looks for the
       crontab file in the current directory unless otherwise specified by
       the command line argument.
"""

__NAME__ = 'sabnzbd.cron'

import time
import string
import logging
import sabnzbd

from threading import Thread

def log(msg):
    logging.info("[%s] %s", __NAME__, msg)

def match(value, expr):
    if expr == '*':
        return 1
    values = string.split(expr,',')
    for v in values:
        if int(v) == value:
            return 1
    return 0

def calcNextTime():
    # Calculate next time with exact seconds.
    tt = time.localtime(time.time() + 60)
    tt = tt[:5] + (0,) + tt[6:]
    return time.mktime(tt)


class CronThread(Thread):
    def __init__(self, cronlines):
        Thread.__init__(self)
        self.cronlines = cronlines
        self.shutdown = False
    
    def stop(self):
        self.shutdown = True
        
    def run(self):
        nextTime = calcNextTime()
        while 1:
            # Sleep for the time remaining until the next exact minute
            currentTime = time.time()
            
            if currentTime < nextTime:
                # Add a fraction of a second to make sure we are in
                # the right second (just for prettier logs)
                sleeptime = (nextTime - currentTime) / 60.0
                for i in xrange(60):
                    if self.shutdown:
                        break
                    time.sleep(sleeptime)
                if self.shutdown:
                    log('%s teared down' % self)
                    break
            # Check if the time has changed by more than two minutes. This
            # case arises when the system clock is changed. We must reset the
            # timer.
            if abs(time.time() - nextTime) > 120:
                log('System clock adjusted? Reseting timer.')
                nextTime = calcNextTime()
                
            # Build a tuple with the current time
            curTuple = time.localtime(nextTime)

            # Read the crontab file
            try:
                for line in self.cronlines:
                    try:
                        tokens = string.split(line)
                        # See if the cron entry matches the current time
                        # minute
                        timeMatch = match(curTuple[4], tokens[0])
                        # hour
                        timeMatch = timeMatch and match(curTuple[3], tokens[1])
                        # day
                        timeMatch = timeMatch and match(curTuple[2], tokens[2])
                        # month
                        timeMatch = timeMatch and match(curTuple[1], tokens[3])
                        # weekday (in crontab 0 or 7 is Sunday)
                        weekday = curTuple[6]+1
                        matchWeekday = match(weekday, tokens[4]) or \
                                          (weekday == 7 and match(0, tokens[4]))
                        timeMatch = timeMatch and matchWeekday
                            
                        if timeMatch:
                            print tokens[5]
                            if tokens[5] == 'pause':
                                log("pausing")
                                sabnzbd.pause_downloader()
                            elif tokens[5] == 'resume':
                                log("resuming")
                                sabnzbd.resume_downloader()
                    except:
                        log('cron: error parsing line %s' % (line))
            except:
                logging.exception("[%s] Unknown Exception", __NAME__)
                
            # Calculate the time for the next iteration (60 seconds later)
            nextTime = nextTime + 60
