#!/usr/bin/python -OO
# Copyright 2005 Gregor Kaufmann <tdn@gmx.at>
# Licensed under the Academic Free License 2.1
"""
sabnzbd.misc - misc classes
"""
__NAME__ = "sabnzbd.misc"

import os
import time
import logging
import sabnzbd
import cherrypy
import urllib

from threading import *
from sabnzbd.nzbstuff import NzbObject
from sabnzbd import nzbgrab

#------------------------------------------------------------------------------

class DirScanner(Thread):
    def __init__(self, dirscan_dir, repair, unpack, delete):
        Thread.__init__(self)
        
        self.dirscan_dir = dirscan_dir
        
        self.r = repair
        self.u = unpack
        self.d = delete
        
        self.shutdown = False
        
    def stop(self):
        logging.info('[%s] Dirscanner shutting down', __NAME__)
        self.shutdown = True
        
    def run(self):
        logging.info('[%s] Dirscanner starting up', __NAME__)
        
        while not self.shutdown:
            time.sleep(1.0)
            try:
                files = os.listdir(self.dirscan_dir)
                
                for filename in files:
                    path = os.path.join(self.dirscan_dir, filename)
                    root, ext = os.path.splitext(path)
                    if ext == '.NZB' or ext == '.nzb':
                        logging.info('[%s] Trying to import %s', __NAME__, path)
                        f = open(path, 'rb')
                        data = f.read()
                        f.close()
                        os.remove(path)
                        sabnzbd.add_nzo(NzbObject(filename, data, self.r, 
                                                  self.u, self.d))
                        sabnzbd.backup_nzb(filename, data)
            except:
                logging.exception("Error importing")
                
class MSGIDGrabber(Thread):
    def __init__(self, nzbun, nzbpw, future_nzo):
        Thread.__init__(self)        
        self.nzbun = nzbun
        self.nzbpw = nzbpw
        self.future_nzo = future_nzo
        
    def run(self):
        try:
            filename, data = nzbgrab.grabnzb(self.future_nzo.postid, self.nzbun,
                                             self.nzbpw)
            if filename and data:
                r, u, d = self.future_nzo.get_repair_opts()
                sabnzbd.insert_future_nzo(self.future_nzo, NzbObject(filename, 
                                                                   data, r,u,d))
                                                                   
                sabnzbd.backup_nzb(filename, data)
            else:
                sabnzbd.remove_nzo(self.future_nzo.nzo_id, False)
        except:
            logging.exception("[%s] Error fetching id %s", 
                              __NAME__, self.future_nzo.postid)
            sabnzbd.remove_nzo(self.future_nzo.nzo_id, False)

class URLGrabber(Thread):
    def __init__(self, url, future_nzo):
        Thread.__init__(self)
        self.url = url
        self.future_nzo = future_nzo
    
    def run(self):
        try:
            opener = urllib.FancyURLopener({})
            opener.prompt_user_passwd = None
            fn, header = opener.retrieve(self.url)
            
            filename, data = (None, None)
            f = open(fn, 'r')
            data = f.read()
            f.close()
            os.remove(fn)
            
            for tup in header.items():
                for item in tup:
                    if "filename=" in item:
                        filename = item[item.index("filename=") + 9:]
                        break
                        
            if data:
                if not filename:
                     filename = os.path.basename(self.url)
                     
                r, u, d = self.future_nzo.get_repair_opts()
                sabnzbd.insert_future_nzo(self.future_nzo, NzbObject(filename, 
                                                                   data, r,u,d))
                sabnzbd.backup_nzb(filename, data)
                
            else:
                sabnzbd.remove_nzo(self.future_nzo.nzo_id, False)
                
        except:
            logging.exception("[%s] Error adding url %s", __NAME__, self.url)
            sabnzbd.remove_nzo(self.future_nzo.nzo_id, False)
